<?php
define('KEYSTORE_BIN','/usr/local/esia/keystore/esia_keystore');
define('CIPHER_FILES_DIR','/usr/local/esia/public_html/data/auth');
define('CLEAR_FILES_DIR','/tmp/auth');
define('ESIA_CONF_FILE','/usr/local/esia/keystore/keystore.conf.php');

include_once ESIA_CONF_FILE;

@mkdir(CIPHER_FILES_DIR,0777,true);
@mkdir(CLEAR_FILES_DIR,0777,true);
chown(CLEAR_FILES_DIR,'www-data');

class KeyStoreException extends Exception{}
class CommandLineException extends KeystoreException{}
class NoDataException extends KeystoreException{}
class UnknownException extends KeystoreException{}
class UnableToDecrypt extends KeystoreException{}

function keystore($key,$action,$mode,$in,$out=null){
        $pre_cmd='';
        $post_cmd='';

	$err_file = $tmpfname = tempnam("/tmp", "err");
        $cmd = KEYSTORE_BIN." -a $action -m $mode -k '$key' 2>$err_file";

        $in_file = explode(':',$in);
        if( $in_file[0] == 'file' ) {
                $pre_cmd = 'cat '.$in_file[1] . ' |';
        }
        else {
                $pre_cmd = "echo '$in'|";
        }

        if( $out != null ) {
                $out_file = explode(':',$out);
                if( $out_file[0] == 'file'){
                        $post_cmd=' > '.$out_file[1];
                }
        }

        $run = "$pre_cmd$cmd$post_cmd";
        exec($run,$output,$ret);

	$err = file_get_contents($err_file);
	unlink($err_file);

	switch($ret){
		case 0 : return implode("\n",$output);
		case 1 : throw new CommandLineException("Error into command line : $err");
		case 2 : throw new NoDataException("No data into input : $err");
		case 3 : throw new UnableToDecrypt("Can't decrypt : $err");
		case -1 : throw new UnknownException("Unknown Error : $err");
	}
}

function getEncryptedUserKey($dbconnection){
	$sql = "SELECT config_value FROM esia_configuration WHERE config_key='USER_KEY'";
	$sth = $dbconnection->prepare($sql);
	$result = $sth->execute();
	return $sth->fetch(PDO::FETCH_OBJ)->config_value;	
}

function setEncryptedUserKey($dbconnection,$key){
	$sql = "UPDATE esia_configuration SET config_value=? WHERE config_key='USER_KEY'";
	$sth = $dbconnection->prepare($sql);
        $sth->execute(array($key));
}

function getConfKey(){
	return KEYSTORE_KEY;
}

function getFiles($dir_path,$files_ext,$child=false){
	$ret = array();	
	$in_dir = scandir($dir_path);

	array_shift($in_dir);//Suppression du .
	array_shift($in_dir);//Suppression du ..
	foreach($in_dir as $file){
		if( is_file("$dir_path/$file")){
			$tmp = explode('.',$file);
			$nb = count($tmp);
			$ext = array_pop($tmp);

			if ($nb > 1 && $ext == $files_ext ){
				 $ret[] = array(
                                'full_path' => "$dir_path/$file",
				'full_dir' => dirname("$dir_path/$file"),
                                'name' => implode('.',$tmp),
                                'full_name' => $file
                        	);
			}
		}
		else if( is_dir("$dir_path/$file") ){
			$ret = array_merge($ret,getFiles("$dir_path/$file",$files_ext,true));
		}
	}
	if($child) return $ret;

	foreach($ret as $idx => $row){
		preg_match("#$dir_path/?(.*)#",$row['full_dir'],$matches);
		$ret[$idx]['dir'] = $matches[1];
	}
	return $ret;
}

function encryptUserKey($conf_key,$decrypted_user_key){
	return keystore($conf_key,'encrypt','user_key',$decrypted_user_key);
}

function decryptUserKey($conf_key,$encrypted_user_key){
	return keystore($conf_key,'decrypt','user_key',$encrypted_user_key);
}

function saveData($user_key,$filepath,$decrypted_data){
	$filepath = CIPHER_FILES_DIR."/$filepath.auth";
	@mkdir(dirname($filepath));
	chown(dirname($filepath),'www-data');
	keystore($user_key,'encrypt','data',$decrypted_data,"file:$filepath");
	chown($filepath,'www-data');
}

function loadData($user_key,$filepath){
        $filepath = CIPHER_FILES_DIR."/$filepath.auth";
        return keystore($user_key,'decrypt','data',"file:$filepath");
}

//DEPRECATED FUNCTIONS
function updateDBClearFilePath($dbconnection,$settings_id,$d_file_path){
 	$sql = "UPDATE esia_device_advance_config SET adv_plugin_keyword_value=? WHERE adv_id=?";
        $sth = $dbconnection->prepare($sql);
        $sth->execute(array($d_file_path,$settings_id));
}

function getDBClearFilePath($dbconnection,$settings_id){
	$sql = "SELECT adv_plugin_keyword_value FROM esia_device_advance_config WHERE adv_id=?";
        $sth = $dbconnection->prepare($sql);
        $result = $sth->execute(array($settings_id));
        return $sth->fetch(PDO::FETCH_OBJ)->adv_plugin_keyword_value;
}

function updateDBGenericFilename($dbconnection,$settings_id,$filename){
        $sql = "UPDATE esia_device_advance_config SET adv_authfile_crypt=? WHERE adv_id=?";
        $sth = $dbconnection->prepare($sql);
        $sth->execute(array($filename,$settings_id));
}

function getDBGenericFileName($dbconnection,$settings_id){
    	$sql = "SELECT adv_authfile_crypt FROM esia_device_advance_config WHERE adv_id=?";
        $sth = $dbconnection->prepare($sql);
        $result = $sth->execute(array($settings_id));
        return $sth->fetch(PDO::FETCH_OBJ)->adv_authfile_crypt;
}
//END DEPRECATED FUNCTIONS

//START => NEW
function removeData($filepath){
	$filepath = CIPHER_FILES_DIR."/$filepath.auth";
	$ret = @unlink($filepath);	
	@rmdir(dirname($filepath));	 //Supprime le repertoire uniquement si il est vide
	return $ret;
}

function isExistFile($filepath){
	$e_filepath = CIPHER_FILES_DIR."/$filepath.auth";
	return is_file ( $e_filepath );
}

function saveSettingsInFiles($dbconnection,$filepath,$decrypted_data,$force=false){
        $e_filepath = CIPHER_FILES_DIR."/$filepath.auth";
        $d_filepath = CLEAR_FILES_DIR."/$filepath.dat";
        $d_fulldir  = dirname($d_filepath);

	if( !$force && isExistFile($filepath)){
		throw new Exception("Unable to save data because $e_filepath file already exist");
	}

	$conf_key =  getConfKey();
	$e_uk=getEncryptedUserKey($dbconnection);
	$d_uk=decryptUserKey($conf_key,$e_uk);

	//Ciphered file save
	saveData($d_uk,$filepath,$decrypted_data);

	//Clear file save
	@mkdir($d_fulldir);
	file_put_contents($d_filepath,"$decrypted_data\n");
}

 //avec force=true peut remplacer la fonction updateSettings mais demande plus de parametres
function insertSettings($dbconnection,$settings_id,$genericfilepath,$decrypted_data,$force=false){ 
	saveSettingsInFiles($dbconnection,$genericfilepath,$decrypted_data,$force);
	updateDBClearFilePath($dbconnection,$settings_id,CLEAR_FILES_DIR."/$genericfilepath.dat");
	updateDBGenericFilename($dbconnection,$settings_id,$genericfilepath);
}

function updateSettings($dbconnection,$settings_id,$decrypted_data){
	$filepath=getDBGenericFilename($dbconnection,$settings_id);
	if(empty($filepath)) throw new Exception('Unable to found file path into DB');
	saveSettingsInFiles($dbconnection,$filepath,$decrypted_data,true);
}

function removeSettings($dbconnection,$settings_id){
	$filepath=getDBGenericFilename($dbconnection,$settings_id);
	return removeData($filepath);
}

function loadSettings($dbconnection,$settings_id){
	$filepath=getDBClearFilePath($dbconnection,$settings_id);
	return file_get_contents($filepath);
}

//STOP => NEW

function decrypt_dir($key){
	$src_dir=CIPHER_FILES_DIR;
	$dst_dir=CLEAR_FILES_DIR;

	$sources = getFiles($src_dir,'auth');
	print_r($sources);
	foreach($sources as $source){
		$dst_full_dir = "$dst_dir/".$source['dir'];
		$dst_full_path = "$dst_full_dir/".$source['name'].'.dat';
		
		@mkdir($dst_full_dir);
		keystore($key,'decrypt','data','file:'.$source['full_path'],"file:$dst_full_path");

                chown($dst_full_dir,'www-data');
		chown($dst_full_path,'www-data');
	}
}

function encrypt_dir($key){
        $src_dir=CLEAR_FILES_DIR;
        $dst_dir=CIPHER_FILES_DIR;

	$sources = getFiles($src_dir,'dat');
	print_r($sources);
        foreach($sources as $source){
                $dst_full_dir = "$dst_dir/".$source['dir'];     
                $dst_full_path = "$dst_full_dir/".$source['name'].'.auth';

                @mkdir($dst_full_dir);
                keystore($key,'encrypt','data','file:'.$source['full_path'],"file:$dst_full_path");

		chown($dst_full_dir,'www-data');
                chown($dst_full_path,'www-data');
        }
}

function change_key_dir($old,$new){
	decrypt_dir($old,CIPHER_FILES_DIR,CLEAR_FILES_DIR);
	encrypt_dir($new,CLEAR_FILES_DIR,CIPHER_FILES_DIR);
}
